@@ -4,6 +4,8 @@ use fixed::traits::FixedSigned;
 extern crate alloc;
 use alloc::vec::Vec;
 
+use cordic;
+
 use num::complex::Complex;
 
 /// Check if x is a power of two.
@@ -61,16 +63,7 @@ pub fn log2( x: usize ) -> usize
 /// 
 /// * `arr` - A mutable reference to the array to do the computation on, and store the result in.
 /// 
-/// ## Example
-/// 
-/// ```
-/// use fixed_trigonometry::fft::*;
-///
-/// let mut arr =  vec![0,1,2,3,4,5,6,7];
-/// bitreverse_array_order::<i32>(&mut arr);
-/// assert_eq!( arr, [0,4,2,6,1,5,3,7] );
-/// ```
-pub fn bitreverse_array_order<T>( arr: &mut Vec<T> )
+fn bitreverse_order<T>( arr: &mut Vec<T> )
     where T: core::marker::Copy
 {
     let n:usize = arr.len();
@@ -100,66 +93,13 @@ pub fn bitreverse_array_order<T>( arr: &mut Vec<T> )
     }
 }
 
-/// Calculating revers number
-fn _reverse( n_capital: usize, n: usize ) -> usize
-{
-  let mut p: usize = 0;
-  
-    for j in 1..=log2(n_capital)
-    {
-        if(n & (1 << (log2(n_capital) - j))) != 0
-        {
-            p |= 1 << (j - 1);
-        }
-    }
-  return p;
-}
-
-/// Using the reverse order in the array
-fn _order<T>( f1: &mut Vec<Complex<T>> ) 
-    where T: Copy
-{
-    // Create a copy of f1.
-    let mut f2 = Vec::<Complex<T>>::with_capacity( f1.len() );
-
-    for i in 0..f1.len()
-    {
-        f2.push( f1[_reverse(f1.len(), i)] );
-    }
-    for j in 0..f1.len()
-    {
-        f1[j] = f2[j];
-    }    
-}
-
-
-/// Butterfly computation for decimate-in-frequeny.
+/// Calculate the Raddix-2 FFT for fixed point vectors.
+/// - Scaled for each butterfly computation.
+/// - Requires input size to be a power of two.
+/// - Computed-in-place.
+/// - Decimation-in-freqency.
 /// 
-/// ## Arguments
-/// 
-/// * `a` - input/output.
-/// * `b` - input/output.
-/// * `w` - twiddle factor.
-/// 
-fn butterfly_df<T>( a: &mut Complex<T>, b: &mut Complex<T>, w:Complex<T> )
-    where T: FixedSigned
-{
-    let temp_a = crate::complex::add(*a,*b);
-    //  let temp_b = complex::mul_cartesian(complex::sub(*a, complex::scale_cartesian(T::from_num(2), *b)), w);
-    let temp_b = crate::complex::mul_cartesian(crate::complex::sub(*a, *b), w);
-    
-    *a = temp_a;
-    *b = temp_b;
-}
-
-/// Calculate the Raddix-2 FFT for fixed point numbers.
-/// Scaled for each butterfly computation.
-/// Requires input size to be a power of two.
-/// 
-/// Computed-in-place.
-/// Decimation-in-freqency.
-/// 
-/// The method utilizes fixed point approximations for square root, sine, cosine and atan calculations.
+/// The function computes the twiddle factor each time it is called, which is suboptimal for repeating computations.
 /// 
 /// ## Arguments
 /// 
@@ -171,7 +111,7 @@ fn butterfly_df<T>( a: &mut Complex<T>, b: &mut Complex<T>, w:Complex<T> )
 /// use fixed_trigonometry::fft::*;
 /// 
 /// use fixed::FixedI32 as F;
-/// use fixed::types::extra::U22 as U;
+/// use fixed::types::extra::U28 as U;
 /// use num::complex::Complex;
 /// 
 /// const N:usize = 4;
@@ -180,13 +120,85 @@ fn butterfly_df<T>( a: &mut Complex<T>, b: &mut Complex<T>, w:Complex<T> )
 /// arr[3].re = F::<U>::from_num(0);
 /// 
 /// fft( &mut arr );
-/// assert_eq!( arr, vec![  Complex::<F<U>>::new(F::<U>::from_num(0.75),       F::<U>::from_num(0)          ),
-///                         Complex::<F<U>>::new(F::<U>::from_num(-0.0100546), F::<U>::from_num(-0.2503753) ),
-///                         Complex::<F<U>>::new(F::<U>::from_num(0.250376),   F::<U>::from_num(0.0)        ),
-///                         Complex::<F<U>>::new(F::<U>::from_num(0.0145547),  F::<U>::from_num(0.2507293)  )] );
+/// assert_eq!( arr, vec![  Complex::<F<U>>::new(F::<U>::from_num(0.75),            F::<U>::from_num(0)            ),
+///                         Complex::<F<U>>::new(F::<U>::from_num(-0.000000004,),   F::<U>::from_num(-0.250376098) ),
+///                         Complex::<F<U>>::new(F::<U>::from_num(0.250376098),     F::<U>::from_num(0.0)          ),
+///                         Complex::<F<U>>::new(F::<U>::from_num(-0.000000007),    F::<U>::from_num(0.25073779)   )] );
 /// ```
 pub fn fft<T>( vec: &mut Vec<Complex<T>> )
-    where T: FixedSigned
+    where T: FixedSigned + cordic::CordicNumber
+{
+    // Process fft.
+    fft_processor(vec, T::from_num(1));
+
+    // Decimation-in-freqency.
+    bitreverse_order(vec); // Bitreverse order
+}
+
+/// Calculate the Raddix-2 Inverse FFT for fixed point vectors.
+/// - Scaled for each butterfly computation.
+/// - Requires input size to be a power of two.
+/// - Computed-in-place.
+/// - Decimation-in-freqency.
+/// 
+/// The function computes the twiddle factor each time it is called, which is suboptimal for repeating computations.
+/// 
+/// ## Arguments
+/// 
+/// * `vec` - A mutable reference to the vector to do the computation on, and store the result in.
+/// 
+/// ## Example
+/// 
+/// ```
+/// use fixed_trigonometry::fft::*;
+/// 
+/// use fixed::FixedI32 as F;
+/// use fixed::types::extra::U28 as U;
+/// use num::complex::Complex;
+/// 
+/// const N:usize = 4;
+/// let mut arr  = vec![ Complex::<F<U>>::new(F::<U>::from_num(1), F::<U>::from_num(0) ); N  ];
+///
+/// arr[3].re = F::<U>::from_num(0);
+/// 
+/// ifft( &mut arr );
+/// assert_eq!( arr, vec![  Complex::<F<U>>::new(F::<U>::from_num(0.75),         F::<U>::from_num(0.0)      ),
+///                         Complex::<F<U>>::new(F::<U>::from_num(-0.000000004), F::<U>::from_num(0.2503761)),
+///                         Complex::<F<U>>::new(F::<U>::from_num(0.250376098),  F::<U>::from_num(0.0)      ),
+///                         Complex::<F<U>>::new(F::<U>::from_num(0.00000001),   F::<U>::from_num(-0.2507378) )] );
+/// ```
+pub fn ifft<T>( vec: &mut Vec<Complex<T>> )
+    where T: FixedSigned + cordic::CordicNumber
+{
+    // Process fft.
+    fft_processor(vec, T::from_num(-1));
+    // Decimation-in-freqency.
+    bitreverse_order(vec); // Bitreverse order
+}
+
+/// Butterfly computation for decimate-in-frequeny.
+/// 
+/// ## Arguments
+/// 
+/// * `a` - input/output.
+/// * `b` - input/output.
+/// * `w` - twiddle factor.
+/// 
+fn butterfly_df<T>( a: &mut Complex<T>, b: &mut Complex<T>, w:Complex<T> )
+    where T: FixedSigned + cordic::CordicNumber
+{
+    let temp_a = crate::complex::add(*a,*b);
+    //  let temp_b = complex::mul_cartesian(complex::sub(*a, complex::scale_cartesian(T::from_num(2), *b)), w);
+    let temp_b = crate::complex::mul_cartesian(crate::complex::sub(*a, *b), w);
+    
+    *a = temp_a;
+    *b = temp_b;
+}
+
+/// Shared fft processor for fft and ifft.
+/// Requires bit-reversion afterwards.
+fn fft_processor<T>( vec: &mut Vec<Complex<T>>, dir: T )
+    where T: FixedSigned + cordic::CordicNumber
 {
     let n = vec.len();
 
@@ -196,12 +208,15 @@ pub fn fft<T>( vec: &mut Vec<Complex<T>> )
     // Calculate Twiddle factor W.
     w.push( Complex::new( <T>::from_num(1), <T>::from_num(0) ) );
 
-    let angle:T = (-<T>::from_num(fixed::consts::TAU)) >> log2(n) as u32;
-    for i in 1..n/2
+    let angle:T = (dir*-<T>::from_num(fixed::consts::TAU)) >> log2(n) as u32;
+    let mut phase_inc = angle;
+    for _i in 1..n/2
     {
         // Calculate twiddle factor for W_i.
-        let real = crate::cos( <T>::from_num(i) *angle );
-	    let imag = crate::sin( <T>::from_num(i) *angle );
+        
+        let (imag, real) = cordic::sin_cos( phase_inc );
+
+        phase_inc = phase_inc+angle;
 
         w.push( Complex::new( real, imag ) );
     }
@@ -240,8 +255,4 @@ pub fn fft<T>( vec: &mut Vec<Complex<T>> )
         num_blocks = num_blocks * 2;
         num_butt   = num_butt / 2;
     }
-
-    // Decimation-in-freqency.
-    //order(vec);    // Bitreverse order
-    bitreverse_array_order(vec); // Bitreverse order
 }
\ No newline at end of file
